package com.payu.reports;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.security.GeneralSecurityException;
import java.security.Key;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import javax.net.ssl.HttpsURLConnection;

public class PayUReports {

	private String apiEndpoint;

	private String merchantCode;

	private Key merchantKey;

	private DateFormat dateFormatter;

	public PayUReports() {
		dateFormatter = new SimpleDateFormat("yyyy-MM-dd");
	}

	public PayUReports setApiEndpoint(String url) {
		apiEndpoint = url;
		return this;
	}

	public PayUReports setSecretKey(String key) {
		merchantKey = new SecretKeySpec(key.getBytes(), "HmacMD5");
		return this;
	}

	public PayUReports setMerchant(String code) {
		merchantCode = code;
		return this;
	}

	public String getOrdersReport(Date startDate, Date endDate) {
		Map<String, String> parameters = new LinkedHashMap<String, String>();

		parameters.put("merchant", merchantCode);
		parameters.put("startDate", dateFormatter.format(startDate));
		parameters.put("endDate", dateFormatter.format(endDate));
		parameters.put("timeStamp",
				Integer.toString((int) (System.currentTimeMillis() / 1000)));
		parameters.put("signature", sign(parameters));

		return callApiForReport("orders", parameters);
	}

	public String getProductsReport(Date startDate, Date endDate) {
		Map<String, String> parameters = new LinkedHashMap<String, String>();

		parameters.put("merchant", merchantCode);
		parameters.put("startDate", dateFormatter.format(startDate));
		parameters.put("endDate", dateFormatter.format(endDate));
		parameters.put("timeStamp",
				Integer.toString((int) (System.currentTimeMillis() / 1000)));
		parameters.put("signature", sign(parameters));

		return callApiForReport("products", parameters);
	}

	public String getOperatorReport(Date startDate, Date endDate) {
		Map<String, String> parameters = new LinkedHashMap<String, String>();

		parameters.put("merchant", merchantCode);
		parameters.put("startDate", dateFormatter.format(startDate));
		parameters.put("endDate", dateFormatter.format(endDate));
		parameters.put("timeStamp",
				Integer.toString((int) (System.currentTimeMillis() / 1000)));
		parameters.put("signature", sign(parameters));

		return callApiForReport("operator", parameters);
	}

	public String getOperatorReportDownloadLink(String operatorReportId) {
		Map<String, String> parameters = new LinkedHashMap<String, String>();

		parameters.put("merchant", merchantCode);
		parameters.put("timeStamp",
				Integer.toString((int) (System.currentTimeMillis() / 1000)));
		parameters.put("signature", sign(parameters));

		return apiEndpoint + "operator/" + operatorReportId + "?"
				+ buildHttpQuery(parameters);
	}

	private String callApiForReport(String report, Map<String, String> parameters) {
		String reportUrl = apiEndpoint + report + "?" + buildHttpQuery(parameters);
		HttpsURLConnection con = null;
		try {
			con = (HttpsURLConnection) (new URL(reportUrl).openConnection());
			con.setRequestMethod("GET");
			BufferedReader rd = new BufferedReader(new InputStreamReader(
					con.getInputStream(), "UTF-8"));
			String line;
			StringBuilder response = new StringBuilder();
			while ((line = rd.readLine()) != null) {
				response.append(line);
				response.append("\n");
			}
			rd.close();
			return response.toString();
		} catch (Exception e) {
			return null;
		} finally {
			if (con != null) {
				con.disconnect();
			}
		}
	}

	private String buildHttpQuery(Map<String, String> parameters) {
		StringBuilder query = new StringBuilder();
		try {
			for (Map.Entry<String, String> entry : parameters.entrySet()) {
				query.append(URLEncoder.encode(entry.getKey(), "UTF-8") + "=");
				query.append(URLEncoder.encode(entry.getValue(), "UTF-8") + "&");
			}
		} catch (UnsupportedEncodingException e) {
		}
		return query.substring(0, query.length() - 1);
	}

	private String sign(Map<String, String> parameters) {
		return createSignature(buildSourceString(parameters));
	}

	private String buildSourceString(Map<String, String> parameters) {
		StringBuilder hashString = new StringBuilder();

		for (Map.Entry<String, String> entry : parameters.entrySet()) {
			hashString.append(Integer.toString(entry.getValue().length()))
					.append(entry.getValue());
		}
		return hashString.toString();
	}

	private String createSignature(String post) {
		try {
			Mac hmac = Mac.getInstance("HmacMD5");
			hmac.init(merchantKey);
			byte[] result = hmac.doFinal(post.getBytes(Charset.forName("UTF-8")));

			StringBuilder resultSb = new StringBuilder();
		    for (byte b : result) {
		    	resultSb.append(String.format("%02x", b));
		    }
		    
			return resultSb.toString();
		} catch (GeneralSecurityException $ex) {
			return "";
		}
	}

}
